package bou.amine.apps.readerforselfossv2.service

import com.russhwolf.settings.Settings
import io.github.aakira.napier.Napier
import io.ktor.client.plugins.*

class ApiDetailsService {
    val settings: Settings = Settings()
    private var _apiVersion: Int = -1
    private var _baseUrl: String = ""
    private var _userName: String = ""
    private var _password: String = ""
    private var _itemsNumber: Int? = null
    private var _apiTimeout: Long? = null

    fun logApiCalls(message: String) {
        Napier.d(message, tag = "LogApiCalls")
    }


    fun getApiVersion(): Int {
        if (_apiVersion == -1) {
            refreshApiVersion()
            return _apiVersion
        }
        return _apiVersion
    }

    fun refreshApiVersion() {
        _apiVersion = settings.getInt("apiVersionMajor", -1)
    }

    fun getBaseUrl(): String {
        if (_baseUrl.isEmpty()) {
            _baseUrl = settings.getString("url", "")
        }
        return _baseUrl
    }

    fun getUserName(): String {
        if (_userName.isEmpty()) {
            _userName = settings.getString("login", "")
        }
        return _userName
    }

    fun getPassword(): String {
        if (_password.isEmpty()) {
            _password = settings.getString("password", "")
        }
        return _password
    }

    fun getItemsNumber(): Int {
        if (_itemsNumber == null) {
            refreshItemsNumber()
        }
        return _itemsNumber!!
    }

    fun refreshItemsNumber() {
        _itemsNumber = settings.getString("prefer_api_items_number", "200").toInt()
    }

    fun getApiTimeout(): Long {
        if (_apiTimeout == null) {
            refreshApiTimeout()
        }
        return _apiTimeout!!
    }

    fun refreshApiTimeout() {
        val settingsTimeout = settings.getLong("api_timeout", HttpTimeout.INFINITE_TIMEOUT_MS)
        _apiTimeout = if (settingsTimeout > 0) settingsTimeout else HttpTimeout.INFINITE_TIMEOUT_MS
    }

    fun refresh() {
        _password = settings.getString("password", "")
        _userName = settings.getString("login", "")
        _baseUrl = settings.getString("url", "")
        refreshApiVersion()
        refreshItemsNumber()
        refreshApiTimeout()
    }

    companion object {
        const val translationUrl = "https://crwd.in/readerforselfoss"

        const val sourceUrl = "https://gitea.amine-louveau.fr/Louvorg/ReaderForSelfoss-multiplatform"

        const val trackerUrl = "https://gitea.amine-louveau.fr/Louvorg/ReaderForSelfoss-multiplatform/issues"

        const val syncChannelId = "sync-channel-id"

        const val newItemsChannelId = "new-items-channel-id"
    }
}